{% extends 'layout.html.twig' %}

{% set topMenu = {
    "menu": [
        { "name": "About", "url": "https://phpdoc.org/"},
        { "name": "Components", "url": "https://phpdoc.org/components.html"},
        { "name": "Documentation", "url": "https://docs.phpdoc.org/"},
    ],
    "social": [
        { "iconClass": "fab fa-mastodon", "url": "https://phpc.social/@phpdoc"},
        { "iconClass": "fab fa-github", "url": "https://github.com/phpdocumentor/typeresolver"},
        { "iconClass": "fas fa-envelope-open-text", "url": "https://github.com/orgs/phpDocumentor/discussions"}
    ]
}
%}
